/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * Exercises various info object handling functions with a UNIQUE CHOICE
 * of UNBOUNDED ENCODED OBJECT IDENTIFIER.  Assumed that malloc() always
 * succeeds and returns a valid pointer.  The functions exersized are:
 *
 * ossAddValToEncodedOid()
 * ossAddInfoObject()
 * ossRemoveInfoObject()
 * ossGetInfoObject()
 */

#include "obj2.h"

#define NUMBER_OF_OBJECTS 2

static ERROR_CLASS _v4 = {
    "A",
    {global_chosen},
    1,
    c		/* equal to 2 in this example */
};


ERROR_CLASS	*p[NUMBER_OF_OBJECTS];
void		*q[NUMBER_OF_OBJECTS];


int main(void)
{

    int		 i, err = 0;
    OssGlobal	 world;		/* OSS global structure */
    void	*object;
    OssEncodedOID eoid = {0};
    unsigned char *encodedOID;


    if (err = ossinit(&world, obj2)) {
	printf("Initialization failed with error code %d.\n", err);
	return 1;
    }
    err = ossAsnValToEncodedOid(&world, "{1 2 3 5 6 70}", &eoid);
    if (err) {
	ossPrint(&world, "Error %d is returned by ossAsnValToEncodedOid().\n", err);
	ossterm(&world);
	return 1;
    }
    _v4.code.u.global.length = eoid.length;
    encodedOID = world.mallocp(eoid.length);
    memcpy(encodedOID, eoid.value, eoid.length);

    for (i = 0; i < NUMBER_OF_OBJECTS; i++) {
	ossPrint(&world, "\tAdding object #%d...\n\n", i);
	p[i] = world.mallocp(sizeof(_v4));
	q[i] = world.mallocp(eoid.length);
	_v4.code.u.global.value = q[i];
	memcpy(p[i], &_v4, sizeof(_v4));
	encodedOID[eoid.length - 1]++;	/* make the encoded OID value different */
	memcpy(q[i], encodedOID, eoid.length);
	err = ossAddInfoObject(&world, ErrorSet_OSET, p[i] /*&_v4*/);
	if (err) {
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n", err);
	    break;
	}
	ossPrint(&world, "Added successfully.\n\n");
	ossPrint(&world, "\tGetting the added object...\n\n");
	object = ossGetInfoObject(&world, ErrorSet_OSET, &p[i]->code);
	if (!object)
	    ossPrint(&world, "NULL is returned by ossGetInfoObject() for object #%d.\n", i);
	else if (object != p[i])
	    ossPrint(&world, "Address is returned by ossGetInfoObject() that does not match.\n");
	else {
	    ERROR_CLASS *obj = object;

	    ossPrint(&world, "Object returned successfully.\n\n");
	    ossPrint(&world, "\tAccessing the category & user fields of the info object #%d...\n\n", i);
	    ossPrint(&world, "The category field is %s.\n", obj->category);
	    ossPrint(&world, "The user field is %d.\n\n", obj->user);
	}
    }

    if (!err) {
	ossPrint(&world, "\tTrying to add object #%d again...\n\n", i - 1);
	err = ossAddInfoObject(&world, ErrorSet_OSET, p[i - 1]);
	if (err)
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n\n", err);

	ossPrint(&world, "\tTrying to add object #%d again...\n\n", 0);
	err = ossAddInfoObject(&world, ErrorSet_OSET, p[0]);
	if (err)
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n", err);
	err = 0;
    }

    for (i = NUMBER_OF_OBJECTS - 1; i >= 0; i--) {
	if (!err) {
	    ossPrint(&world, "\n\tRemoving object #%d...\n\n", i);
	    err = ossRemoveInfoObject(&world, ErrorSet_OSET, p[i] /*&_v4*/);
	    if (err)
		ossPrint(&world, "Error %d is returned by ossRemoveInfoObject().\n", err);
	    else
		ossPrint(&world, "Removed successfully.\n", i);
	    if (!err && !i) {
		ossPrint(&world, "\n\tGetting the removed object...\n\n");
		object = ossGetInfoObject(&world, ErrorSet_OSET, &p[i]->code);
		if (!object)
		    ossPrint(&world, "NULL is returned by ossGetInfoObject() for object #%i because it was removed.\n\n", i);
		else if (object != p[i])
		    ossPrint(&world, "Address is returned by ossGetInfoObject() that does not match.\n\n");
		else
		    ossPrint(&world, "Object returned successfully in error.\n\n");
	    }
	}
	if (p[i])
	    world.freep(p[i]);
	if (q[i])
	    world.freep(q[i]);
    }
    world.freep(encodedOID);
    ossterm(&world);
    return err;
}
