/*
 * Copyright (C) 2001-2011 OSS Nokalva.  All rights reserved.
 *
 */

/*
 * THIS FILE IS PROPRIETARY MATERIAL OF OSS NOKALVA, INC. AND MAY BE USED
 * ONLY BY DIRECT LICENSEES OF OSS NOKALVA, INC. THIS FILE MAY NOT BE
 * DISTRIBUTED.
 */

/*
 * Exercises various object handling functions with a UNIQUE CHOICE
 * of UNBOUNDED ENCODED OBJECT IDENTIFIER and UNIQUE INTEGER for the RANAP
 * protocol.  Assumed that malloc() always succeeds and returns a valid pointer.
 * The functions exersized are:
 *
 * ossAddInfoObject()
 * ossRemoveInfoObject()
 * ossGetInfoObject()
 */

#include "ranap.h"

#define NUMBER_OF_OBJECTS 2

/*
 * The following information objects are only needed for illustration
 * about how you can add new information objects to the already present
 * in the protocol dynamically at run time.
 *
 */
static RANAP_ELEMENTARY_PROCEDURE my_own1 = {
    SuccessfulOutcome_present | criticality_present,
    2,		/* InitiatingMessage */
    3,		/* SuccessfulOutcome */
    0,		/* UnsuccessfulOutcome, absent here */
    0,		/* Outcome, absent here */
    1,		/* procedureCode, a field marked UNIQUE */
    ignore	/* criticality, "ignore" if absent */
};

static RANAP_PROTOCOL_IES my_own2 = {
    65535,	/* id */
    ignore,	/* criticality */
    93,		/* Value */
    mandatory	/* presence */
};




RANAP_ELEMENTARY_PROCEDURE	*p[NUMBER_OF_OBJECTS];
RANAP_PROTOCOL_IES		*q[NUMBER_OF_OBJECTS];


int main(void)
{

    int		 i, err = 0;
    OssGlobal	 world;		/* OSS global structure */
    void	*object;
    OssEncodedOID eoid = {0};


    if (err = ossinit(&world, ranap)) {
	printf("Initialization failed with error code %d.\n", err);
	return 1;
    }

    ossPrint(&world, "\tRunning the code for the RANAP_ELEMENTARY_PROCEDURE class...\n\n");

    my_own1.procedureCode = 255;

    for (i = 0; i < NUMBER_OF_OBJECTS; i++) {
	ossPrint(&world, "\tAdding object #%d...\n\n", i);
	p[i] = world.mallocp(sizeof(my_own1));
	my_own1.procedureCode--;	/* make the procedure code different */
	memcpy(p[i], &my_own1, sizeof(my_own1));
	err = ossAddInfoObject(&world, RANAP_ELEMENTARY_PROCEDURES_OSET, p[i] /*&my_own1*/);
	if (err) {
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n", err);
	    break;
	}
	ossPrint(&world, "Added successfully.\n\n", i);
	ossPrint(&world, "\tGetting the added object...\n\n");
	object = ossGetInfoObject(&world, RANAP_ELEMENTARY_PROCEDURES_OSET, &p[i]->procedureCode);
	if (!object)
	    ossPrint(&world, "NULL is returned by ossGetInfoObject() for object #%d.\n", i);
	else if (object != p[i])
	    ossPrint(&world, "Address is returned by ossGetInfoObject() that does not match.\n");
	else {
	    RANAP_ELEMENTARY_PROCEDURE *obj = object;

	    ossPrint(&world, "Object returned successfully.\n\n");
	    ossPrint(&world, "\tAccessing the criticality & procedure code fields of the info object #%d...\n\n", i);
	    ossPrint(&world, "The criticality field is %d.\n", obj->criticality);
	    ossPrint(&world, "The procedure code field is %d.\n\n", obj->procedureCode);
	}
    }

    if (!err) {
	ossPrint(&world, "\tTrying to add object #%d again...\n\n", i - 1);
	err = ossAddInfoObject(&world, RANAP_ELEMENTARY_PROCEDURES_OSET, p[i - 1]);
	if (err)
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n\n", err);

	ossPrint(&world, "\tTrying to add object #%d again...\n\n", 0);
	err = ossAddInfoObject(&world, RANAP_ELEMENTARY_PROCEDURES_OSET, p[0]);
	if (err)
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n", err);
	err = 0;
    }

    for (i = NUMBER_OF_OBJECTS - 1; i >= 0; i--) {
	if (!err) {
	    ossPrint(&world, "\n\tRemoving object #%d...\n\n", i);
	    err = ossRemoveInfoObject(&world, RANAP_ELEMENTARY_PROCEDURES_OSET, p[i] /*&my_own1*/);
	    if (err)
		ossPrint(&world, "Error %d is returned by ossRemoveInfoObject().\n", err);
	    else
		ossPrint(&world, "Removed successfully.\n", i);
	    if (!err && !i) {
		ossPrint(&world, "\n\tGetting the removed object...\n\n");
		object = ossGetInfoObject(&world, RANAP_ELEMENTARY_PROCEDURES_OSET, &p[i]->procedureCode);
		if (!object)
		    ossPrint(&world, "NULL is returned by ossGetInfoObject() for object #%i because it was removed.\n\n", i);
		else if (object != p[i])
		    ossPrint(&world, "Address is returned by ossGetInfoObject() that does not match.\n\n");
		else
		    ossPrint(&world, "Object returned successfully in error.\n\n");
	    }
	}
	if (p[i])
	    world.freep(p[i]);
    }

    ossPrint(&world, "\n\tRunning the code for the RANAP_PROTOCOL_IES class...\n\n");

    my_own2.id = 65535;

    for (i = 0; i < NUMBER_OF_OBJECTS; i++) {
	ossPrint(&world, "\tAdding object #%d...\n\n", i);
	q[i] = world.mallocp(sizeof(my_own2));
	my_own2.id--;		/* make the id code different */
	memcpy(q[i], &my_own2, sizeof(my_own2));
	err = ossAddInfoObject(&world, RelocationRequestIEs_OSET, q[i] /*&my_own2*/);
	if (err) {
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n", err);
	    break;
	}
	ossPrint(&world, "Added successfully.\n\n", i);
	ossPrint(&world, "\tGetting the added object...\n\n");
	object = ossGetInfoObject(&world, RelocationRequestIEs_OSET, &q[i]->id);
	if (!object)
	    ossPrint(&world, "NULL is returned by ossGetInfoObject() for object #%d.\n", i);
	else if (object != q[i])
	    ossPrint(&world, "Address is returned by ossGetInfoObject() that does not match.\n");
	else {
	    RANAP_PROTOCOL_IES *obj = object;

	    ossPrint(&world, "Object returned successfully.\n\n");
	    ossPrint(&world, "\tAccessing the id, criticality & presence fields of the info object #%d...\n\n", i);
	    ossPrint(&world, "The id field is %d.\n", obj->id);
	    ossPrint(&world, "The criticality field is %d.\n", obj->criticality);
	    ossPrint(&world, "The presence field is %d.\n\n", obj->presence);
	}
    }

    if (!err) {
	ossPrint(&world, "\tTrying to add object #%d again...\n\n", i - 1);
	err = ossAddInfoObject(&world, RelocationRequestIEs_OSET, q[i - 1]);
	if (err)
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n\n", err);

	ossPrint(&world, "\tTrying to add object #%d again...\n\n", 0);
	err = ossAddInfoObject(&world, RelocationRequestIEs_OSET, q[0]);
	if (err)
	    ossPrint(&world, "Error %d is returned by ossAddInfoObject().\n", err);
	err = 0;
    }

    for (i = NUMBER_OF_OBJECTS - 1; i >= 0; i--) {
	if (!err) {
	    ossPrint(&world, "\n\tRemoving object #%d...\n\n", i);
	    err = ossRemoveInfoObject(&world, RelocationRequestIEs_OSET, q[i] /*&my_own2*/);
	    if (err)
		ossPrint(&world, "Error %d is returned by ossRemoveInfoObject().\n", err);
	    else
		ossPrint(&world, "Removed successfully.\n", i);
	    if (!err && !i) {
		ossPrint(&world, "\n\tGetting the removed object...\n\n");
		object = ossGetInfoObject(&world, RelocationRequestIEs_OSET, &q[i]->id);
		if (!object)
		    ossPrint(&world, "NULL is returned by ossGetInfoObject() for object #%i because it was removed.\n\n", i);
		else if (object != p[i])
		    ossPrint(&world, "Address is returned by ossGetInfoObject() that does not match.\n\n");
		else
		    ossPrint(&world, "Object returned successfully in error.\n\n");
	    }
	}
	if (q[i])
	    world.freep(q[i]);
    }
    ossterm(&world);
    return err;
}
